import torch
import torch.nn as nn
import torch.optim as optim
import torchaudio
import tqdm

from torch.utils.data import random_split
from torch.utils.data import DataLoader

from data import AudioDataset
from generator import SISUGenerator
from discriminator import SISUDiscriminator

# Check for CUDA availability
device = torch.device("cuda" if torch.cuda.is_available() else "cpu")
print(f"Using device: {device}")

# Initialize dataset and dataloader
dataset_dir = './dataset/good'
dataset = AudioDataset(dataset_dir, target_duration=2.0)  # 5 seconds target duration

dataset_size = len(dataset)
train_size = int(dataset_size * .9)
val_size = int(dataset_size-train_size)

train_dataset, val_dataset = random_split(dataset, [train_size, val_size])

train_data_loader = DataLoader(train_dataset, batch_size=4, shuffle=True)
val_data_loader = DataLoader(val_dataset, batch_size=4, shuffle=True)

# Initialize models and move them to device
generator = SISUGenerator()
discriminator = SISUDiscriminator()

generator = generator.to(device)
discriminator = discriminator.to(device)

# Loss and optimizers
criterion = nn.MSELoss()  # Use Mean Squared Error loss
optimizer_g = optim.Adam(generator.parameters(), lr=0.0005, betas=(0.5, 0.999))
optimizer_d = optim.Adam(discriminator.parameters(), lr=0.0001, betas=(0.5, 0.999))

# Learning rate scheduler
scheduler = torch.optim.lr_scheduler.ReduceLROnPlateau(optimizer_d, mode='min', factor=0.1, patience=5)

# Training loop
num_epochs = 500
for epoch in range(num_epochs):
    latest_crap_audio = torch.empty((2,3), dtype=torch.int64)
    for high_quality, low_quality in tqdm.tqdm(train_data_loader):
        # Check for NaN values in input tensors
        if torch.isnan(low_quality).any() or torch.isnan(high_quality).any():
            continue

        high_quality = high_quality.to(device)
        low_quality = low_quality.to(device)

        batch_size = low_quality.size(0)

        # Labels
        real_labels = torch.ones(batch_size, 1).to(device)
        fake_labels = torch.zeros(batch_size, 1).to(device)

        # Train Discriminator
        optimizer_d.zero_grad()
        outputs = discriminator(high_quality)
        d_loss_real = criterion(outputs, real_labels)
        d_loss_real.backward()

        resampled_audio = generator(low_quality)

        outputs = discriminator(resampled_audio.detach())
        d_loss_fake = criterion(outputs, fake_labels)
        d_loss_fake.backward()


        # Gradient clipping for discriminator
        clip_value = 2.0
        for param in discriminator.parameters():
            if param.grad is not None:
                param.grad.clamp_(-clip_value, clip_value)

        optimizer_d.step()

        d_loss = d_loss_real + d_loss_fake

        # Train Generator
        optimizer_g.zero_grad()
        outputs = discriminator(resampled_audio)
        g_loss = criterion(outputs, real_labels)
        g_loss.backward()

        # Gradient clipping for generator
        clip_value = 1.0
        for param in generator.parameters():
            if param.grad is not None:
                param.grad.clamp_(-clip_value, clip_value)

        optimizer_g.step()

        scheduler.step(d_loss + g_loss)
        latest_crap_audio = resampled_audio

    if epoch % 10 == 0:
        print(latest_crap_audio.size())
        torchaudio.save(f"./epoch-{epoch}-audio.wav", latest_crap_audio[0].cpu(), 44100)
    print(f'Epoch [{epoch+1}/{num_epochs}]')

torch.save(generator.state_dict(), "generator.pt")
torch.save(discriminator.state_dict(), "discriminator.pt")

print("Training complete!")
